<?php

namespace Drupal\sync_clients;

use Drupal\advancedqueue\Entity\Queue;
use Drupal\advancedqueue\Job;
use Drupal\advancedqueue\Plugin\AdvancedQueue\Backend\Database;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityInterface;
use Drupal\sync_clients\Exceptions\QueueNotFoundException;
use Drupal\sync_clients\SyncClient\SyncClientManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Default implementation of QueueHelperInterface.
 */
class QueueHelper implements QueueHelperInterface {

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   * @param \Drupal\sync_clients\SyncClient\SyncClientManagerInterface $syncClientManager
   *   The Sync Client Manager.
   */
  public function __construct(
    protected Connection $database,
    protected LoggerInterface $logger,
    protected SyncClientManagerInterface $syncClientManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function addToQueue(
    string $job_type_id,
    EntityInterface $entity = NULL,
    array $payload_extra = [],
    bool $allow_duplicates = TRUE,
    int $delay = 0,
  ): ?Job {
    $sync_client = $this->syncClientManager->getPluginByJobTypeId($job_type_id);

    $queue_id = $sync_client->getQueueId();
    $queue = Queue::load($queue_id);
    if (is_null($queue)) {
      throw new QueueNotFoundException("Queue $queue_id not found, for Sync Client {$sync_client->getLabel()}");
    }

    $payload = $sync_client->buildQueueItemPayload($entity, $payload_extra);
    if ($allow_duplicates === FALSE) {
      // First check if there is a pending Jobs present for the same entity.
      // If so, we do not need to add it again.
      $payload_keys = $sync_client->duplicateDetectionQueueItemPayloadKeys();
      if ($this->existsInQueue($queue, $job_type_id, $payload, $payload_keys)) {
        return NULL;
      }
    }

    $job = Job::create($job_type_id, $payload);
    $queue->enqueueJob($job, $delay);

    return $job;
  }

  /*
   * Helper functions.
   */

  /**
   * Check if given payload exists in the Queue as pending.
   *
   * @param \Drupal\advancedqueue\Entity\Queue $queue
   *   The queue to query.
   * @param string $job_type_id
   *   The Job Type.
   * @param array $payload
   *   The payload to check.
   * @param array $payload_keys
   *   The payload keys to use for the check.
   *
   * @return bool
   *   TRUE if the payload exists and is still pending.
   *
   * @throws \Exception
   */
  protected function existsInQueue(Queue $queue, string $job_type_id, array $payload, array $payload_keys): bool {
    foreach ($payload_keys as $key) {
      if (!isset($payload[$key])) {
        unset($payload_keys[$key]);
      }
    }

    if (!empty($payload_keys)) {
      $backend = $queue->getBackend();
      if ($backend instanceof Database) {
        $database = \Drupal::database();
        $query = $database->select('advancedqueue', 'a');

        // Get in queue and processing items.
        $orGroup = $query->orConditionGroup()
          ->condition('a.state', Job::STATE_QUEUED)
          ->condition('a.state', Job::STATE_PROCESSING);

        $query->fields('a', ['job_id'])
          ->condition('a.queue_id', $queue->id())
          ->condition('a.type', $job_type_id)
          ->condition($orGroup);

        $condition_group = $query->andConditionGroup();

        foreach ($payload_keys as $key) {
          if (is_int($payload[$key])) {
            $condition_group->where("JSON_EXTRACT(payload, '$.$key') = $payload[$key]");
          }
          else {
            $condition_group->where("JSON_EXTRACT(payload, '$.$key') = '$payload[$key]'");
          }
        }

        $query->condition($condition_group);

        $matches = $query->execute()->fetchCol();
        return !empty($matches);
      }
    }

    return FALSE;
  }

}
